/*
    Description: Provide Gauges for <projectname> (Main Code)
         Author: Tim Brocklehurst
        Version: 0.1.0
           Date: 09 December 2007
        License: GPLv3

Changelog:
    09 December 2007 - v0.1.0 alpha:
        First Release
*/


#include <QtGui>
#include <math.h>
#include "gauge.h"

Gauge::Gauge(QWidget *parent)
           : QWidget(parent)
{
  // Setup Variables
  PI=4.*atan(1.);
  g_antialias=true;
  g_type=0;
  a_type=0;
  s_tics=11;
  s_subtics=5;
  s_int=true;
  g_quality=120;
  g_border=0;
  startAng=30.;
  endAng=330.;
  chev_width=15.;
  min=0.;
  max=100.;
  setValue(13);
  name="Demo Dial";
  setWindowTitle(tr("Circular Gauge"));
  update();
}

void Gauge::paintEvent(QPaintEvent *event){
  // This is the easiest way to implement drawing (and is actually fairly convenient!)
  centre=QPointF(width()/2., height()/2.);
  r=height()/2-g_border;
  if (height()>width()) r=width()/2-g_border;

  if (g_type==0) draw_Plate0();
  if (a_type==0) draw_Arrow0();
}

void Gauge::draw_Plate0(){
  int i;
  double d, l, h;
  double x, y, rt, cr;
  char text[10];
  QPointF *inner_chevron, *outer_chevron;
  QPointF p0, p1, p2;

  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing, g_antialias);

// Draw an artistic gauge with centre at x0,y0
  painter.drawEllipse(centre.x()-r,centre.y()-r,r*2,r*2);

// Allocate memory for chevrons
  inner_chevron = new QPointF[g_quality];
  outer_chevron = new QPointF[g_quality];

// Chevron
  for (i=0;i<g_quality;i++){
    l=(double)i/(double)(g_quality-1);
    d=(endAng-startAng)*l+startAng;
    cr=0.6;
    if (endAng-startAng>360.) cr=.4+l*0.2;
    if (endAng-startAng<-360.) cr=.4-l*0.2;

    x=sin(d*PI/180.)*(cr*(double)r+l*chev_width/2.);
    y=-cos(d*PI/180.)*(cr*(double)r+l*chev_width/2.);
    outer_chevron[i]=centre+QPointF(x,y);

    x=sin(d*PI/180.)*(cr*(double)r-l*chev_width/2.);
    y=-cos(d*PI/180.)*(cr*(double)r-l*chev_width/2.);
    inner_chevron[i]=centre+QPointF(x,y);
  }
  painter.drawPolyline(inner_chevron,g_quality);
  painter.drawPolyline(outer_chevron,g_quality);
  painter.drawLine(inner_chevron[g_quality-1],outer_chevron[g_quality-1]);

  rt=.95;
  for (i=0;i<=s_subtics*(s_tics-1);i++){
    l=(double)i/(double)(s_subtics*(s_tics-1));
    d=(endAng-startAng)*l+startAng;

    x=sin(d*PI/180.)*(double)r;
    y=-cos(d*PI/180.)*(double)r;
    p1=QPointF(x,y);
    p2=p1*rt;
    painter.drawLine(centre+p1,centre+p2);
  }

  rt=.90;
  for (i=0;i<s_tics;i++){
    l=(double)i/(double)(s_tics-1);
    d=(endAng-startAng)*l+startAng;

    x=sin(d*PI/180.)*(double)r;
    y=-cos(d*PI/180.)*(double)r;
    p1=QPointF(x,y);
    p2=p1*rt;
    painter.drawLine(centre+p1,centre+p2);
  }

  for (i=0;i<s_tics;i++){
    l=(double)i/(double)(s_tics-1);
    d=(endAng-startAng)*l+startAng;
    if (!s_int) sprintf(text,"%4.1f",l*(max-min)+min);
    if (s_int)  sprintf(text,"%i",(int)(l*(max-min)+min+.5));
    x=centre.x()+sin(d*PI/180.)*r*.8;
    y=centre.y()-cos(d*PI/180.)*r*.8;
    l=strlen(text)*8;
    h=14;
    painter.drawText((int)x-l/2,(int)y-h/2,l,h,Qt::AlignHCenter || Qt::AlignVCenter,text);
  }

  // We want the text forward of the dial background, but behind the arrow
  painter.drawText(centre.x()-50,centre.y()+r/4,100,20,Qt::AlignHCenter,name);
}

void Gauge::draw_Arrow0()
{
  double d;
  QPointF pt[5];
  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing, g_antialias);

  d=(endAng-startAng)*g_value/max+startAng;

  pt[0]=QPointF(0,0);
  pt[1]=QPointF(0,-0.65*r+4.);
  pt[2]=QPointF(0,-0.65*r);
  pt[3]=QPointF(0,-0.65*r-4.);
  pt[4]=QPointF(0,-0.8*r);

  rotate(pt,5,d);
  translate(pt,5);
  painter.drawEllipse((int)pt[0].x()-2, (int)pt[0].y()-2, 4,4);
  painter.drawLine(pt[0],pt[1]);
  painter.drawEllipse((int)pt[2].x()-4, (int)pt[2].y()-4, 8,8);
  painter.drawLine(pt[3], pt[4]);
}


void Gauge::setPlateType(int val){
  g_type=val;
  update();
  emit changed();
}

int Gauge::plateType(){ return g_type; }

void Gauge::setArrowType(int val){
  a_type=val;
  update();
  emit changed();
}

int Gauge::arrowType(){ return a_type; }

void Gauge::setAntialias(bool val){
  g_antialias=val;
  update();
  emit changed();
}

bool Gauge::antialias(){ return g_antialias; }

void Gauge::setValue(int val){ setValue((double)val); }

void Gauge::setValue(double val){
  g_value=val;
  if (g_value<min) g_value=min;
  if (g_value>max) g_value=max;
  update();
  emit changed();
}

double Gauge::value(){ return g_value; }

int Gauge::value_int(){ return (int)(g_value+.5); }

void Gauge::setMaximum(double val){
  max=val;
  update();
  emit changed();
}

double Gauge::maximum(){ return max; }

void Gauge::setMinimum(double val){
  min=val;
  update();
  emit changed();
}

double Gauge::minimum(){ return min; }

void Gauge::setLabel(QString val){
  name=val;
  update();
  emit changed();
}

QString Gauge::label(){ return name; }

void Gauge::setTics(int val){
  s_tics=val;
  update();
  emit changed();
}

int Gauge::tics(){ return s_tics; }

void Gauge::setSubTics(int val){
  s_subtics=val;
  update();
  emit changed();
}

int Gauge::subTics(){ return s_subtics; }

void Gauge::setStartAngle(double val){
  startAng=val;
  update();
  emit changed();
}

double Gauge::startAngle(){ return startAng; }

void Gauge::setEndAngle(double val){
  endAng=val;
  update();
  emit changed();
}

double Gauge::endAngle(){ return endAng; }

void Gauge::setChevronWidth(double val){
  chev_width=val;
  update();
  emit changed();
}

double Gauge::chevronWidth(){ return chev_width; }

void Gauge::setQuality(int val){
  g_quality=val;
  update();
  emit changed();
}

int Gauge::quality(){ return g_quality; }

void Gauge::setBorder(int val){
  g_border=val;
  update();
  emit changed();
}

int Gauge::border(){ return g_border; }

void Gauge::translate(QPointF pt[], int n){
  int i;
  for (i=0;i<n;i++) pt[i]=centre+pt[i];
}

void Gauge::rotate(QPointF pt[], int n, double theta){
  int i;
  double ix, iy;
  for (i=0;i<n;i++){
    ix=pt[i].x();
    iy=pt[i].y();
    pt[i].setX( ix*cos(-theta*PI/180.)+iy*sin(-theta*PI/180.) );
    pt[i].setY(-ix*sin(-theta*PI/180.)+iy*cos(-theta*PI/180.) );
  }
}
