/*
    Description: Provide Gauges for <projectname> (Class Header)
         Author: Tim Brocklehurst
        Version: 0.1.0
           Date: 09 December 2007
        License: GPLv3

Changelog:
    09 December 2007 - v0.1.0 alpha:
        First Release
*/

#ifndef Gauge_H
  #define Gauge_H
#include <QtGui>
#include <QtDesigner/QDesignerExportWidget>

class QDESIGNER_WIDGET_EXPORT Gauge : public QWidget
{
   Q_OBJECT
   Q_PROPERTY(int Plate READ plateType WRITE setPlateType)
   Q_PROPERTY(int Arrow READ arrowType WRITE setArrowType)
   Q_PROPERTY(bool AntiAliased READ antialias WRITE setAntialias)
   Q_PROPERTY(double Minimum READ minimum WRITE setMinimum)
   Q_PROPERTY(double Maximum READ maximum WRITE setMaximum)
   Q_PROPERTY(double Value READ value WRITE setValue)
   Q_PROPERTY(QString Label READ label WRITE setLabel)
   Q_PROPERTY(int Tics READ tics WRITE setTics)
   Q_PROPERTY(int SubTics READ subTics WRITE setSubTics)
   Q_PROPERTY(double StartAngle READ startAngle WRITE setStartAngle)
   Q_PROPERTY(double EndAngle READ endAngle WRITE setEndAngle)
   Q_PROPERTY(int ChevronWidth READ chevronWidth WRITE setChevronWidth)
   Q_PROPERTY(int Border READ border WRITE setBorder)

public:
  Gauge(QWidget *parent=0);
  double max, min;
  

public slots:
  void setPlateType(int);
  void setArrowType(int);
  int  plateType();
  int  arrowType();
  void setAntialias(bool);
  bool antialias();
  void setValue(int);
  void setValue(double);
  double value();
  int value_int();
  void setMaximum(double);
  double maximum();
  void setMinimum(double);
  double minimum();
  void setLabel(QString);
  QString label();
  void setTics(int);
  int tics();
  void setSubTics(int);
  int subTics();

  void setStartAngle(double);
  double startAngle();
  void setEndAngle(double);
  double endAngle();
  void setChevronWidth(double);
  double chevronWidth();
  void setQuality(int);
  int quality();
  void setBorder(int);
  int border();

signals:
  void changed();

private:
  void paintEvent(QPaintEvent *event);
  void draw_Plate0();
  void draw_Arrow0();
  void translate(QPointF pt[], int n);
  void rotate(QPointF pt[], int n, double theta);

  int g_type, a_type, g_border;
  int s_tics, s_subtics;
  int x0, y0, r, g_quality;
  QPointF centre;
  double g_value, chev_width;
  bool g_antialias, s_int;
  double startAng, endAng, PI;
  QString name;
};

#endif

